---
published: true
layout: post
title: "Cluster"
categories: [MLOps]
tags: [Cluster]
---

## 클러스터란?

- 클러스터는 여러 대의 컴퓨터(혹은 서버)를 하나의 시스템처럼 연결하여 처리 능력을 향상시키고, 높은 가용성, 확장성, 부하 분산 등을 제공하는 기술
- 클러스터는 **하드웨어와 소프트웨어의 결합**으로, 여러 대의 서버나 컴퓨터가 협력하여 작업을 분담하고 처리할 수 있게 만들어짐

## 클러스터의 주요 특징

- **병렬 처리 (Parallel Processing)**  
   여러 대의 컴퓨터가 동시에 작업을 처리하기 때문에, 하나의 시스템보다 훨씬 더 빠르고 효율적으로 대규모 작업을 처리할 수 있음

- **고가용성 (High Availability)**  
   클러스터 내의 서버 중 하나가 장애를 일으킬 경우, 다른 서버가 그 역할을 대신하여 서비스의 중단을 최소화하고 이를 통해 클러스터는 시스템 장애에 강한 구조를 가짐

- **확장성 (Scalability)**  
   클러스터는 수평 확장(horizontal scaling)이 용이함. 즉, 더 많은 서버를 추가하여 시스템의 성능을 향상시킬 수 있고, 이는 클러스터가 매우 유연하게 확장될 수 있음을 의미함

- **부하 분산 (Load Balancing)**  
   클러스터는 여러 서버에 작업을 분배하여 **부하 분산**을 할 수 있음. 이를 통해 성능 최적화가 가능하며, 서버가 과부하되지 않도록 관리할 수 있음

- **자원 공유 (Resource Sharing)**  
   여러 서버가 하나의 시스템처럼 동작하기 때문에, CPU, 메모리, 저장소 등의 자원을 공유하여 활용할 수 있음. 이를 통해 각 서버의 자원을 최적화하여 효율적인 운영이 가능함

## 클러스터의 종류

- **고가용성 클러스터 (High Availability Cluster, HA Cluster)**  
   고가용성 클러스터는 시스템의 **중단을 최소화**하는 것을 목표로 하고, 하나의 서버가 다운되면 다른 서버가 자동으로 그 역할을 대신해 서비스가 계속 운영됨. 이 방식은 주로 **비즈니스 중요 시스템**에서 사용됨

- **부하 분산 클러스터 (Load Balancing Cluster)**  
   여러 서버가 클라이언트의 요청을 분배하여 처리하는 클러스터로 사용자가 서버에 요청을 보내면, 클러스터가 이를 여러 서버에 나누어 처리함. 이 방식은 **웹 서버**나 **데이터베이스** 등의 **트래픽 분산**에 사용됨

- **컴퓨팅 클러스터 (Compute Cluster)**  
   대규모 계산 작업을 분산하여 처리하는 클러스터. 예를 들어, 과학적 연구나 빅데이터 분석 작업에서 **병렬 컴퓨팅**을 활용해 여러 대의 컴퓨터가 **분산 처리**하여 연산 작업을 효율적으로 수행함

- **스토리지 클러스터 (Storage Cluster)**  
   데이터를 여러 서버에 분산 저장하여 고가용성과 높은 성능을 제공하는 클러스터. **분산 파일 시스템**이나 **네트워크 스토리지**를 사용할 수 있고, 이를 통해 **데이터의 중복 저장**과 **고속 접근**을 가능하게 함

## 클러스터의 구성 요소

- **클러스터 노드 (Cluster Node)**  
   클러스터는 여러 대의 **노드**로 구성되어, 각 노드는 독립적인 컴퓨터나 서버이며, 클러스터 내에서 서로 협력하여 작업을 처리함. 노드는 **마스터 노드**와 **워커 노드**로 구분될 수 있음
   - **마스터 노드 (Master Node)**: 클러스터의 관리를 담당하며, 작업의 배분과 모니터링을 수행함
   - **워커 노드 (Worker Node)**: 실제 작업을 수행하는 노드

- **네트워크**  
   클러스터 내 노드들은 **네트워크**를 통해 서로 연결되어야 함. 네트워크는 클러스터의 성능에 큰 영향을 미치므로, **저지연(低遲延)** 네트워크를 사용하여 빠른 데이터 전송과 동기화가 이루어지도록 해야 함

- **클러스터 관리 소프트웨어**  
   클러스터의 **작업 분배**, **자원 할당**, **상태 모니터링** 등을 관리하는 소프트웨어가 필요. 예를 들어, **Kubernetes**는 컨테이너화된 환경에서 **클러스터 관리**를 담당하는 대표적인 툴

- **스토리지 시스템**  
   클러스터는 **분산 스토리지 시스템**을 갖추고 있을 수 있으며, **파일 시스템**이나 **블록 스토리지**를 클러스터 내에서 공유할 수 있도록 설정함

## 클러스터 설계 시 고려사항

- **성능**  
   클러스터의 성능은 각 노드의 하드웨어 사양뿐만 아니라, **네트워크 속도**와 **작업 분배 효율**에 달려 있고, 설계 시 이러한 요소들을 고려해야 함

- **보안**  
   여러 대의 서버가 연결된 클러스터에서는 **네트워크 보안**, **데이터 암호화**, **접근 제어** 등의 보안 요소를 강화해야 함

- **확장성**  
   클러스터는 쉽게 확장할 수 있어야 하므로, 시스템이 커짐에 따라 **자동 확장**(auto-scaling) 기능을 지원하는지 확인해야 함

- **장애 복구**  
   클러스터 내에서 일부 노드가 다운되었을 때 이를 자동으로 복구할 수 있도록 설계해야 하고, **고가용성**을 보장하기 위해 장애 복구 절차를 명확히 해야 함
